import ppcardhandler.GenericDataContainer

class Status(GenericDataContainer.GenericDataContainer):
    def __init__(self, name, id, data=None):
        """
        Corresponds to a Status column in a Board in Projectplace
        @param name: Name of Status
        @type name: '<name>'
        @param id: Id of Status
        @type id: <id>
        @param data: Data retrieved from Projectplace API representing a Status
        @type data: {'name':'<name>','id':<id>, ...} [primary data, more will be available]
        """
        self.name = name
        self.id = id
        
        if data:
            data2 = data.copy()
            del data2['id']     # We don't want these to overwrite the self.id
            #del data2['name']  # There is no "name" key, only "title".
            GenericDataContainer.GenericDataContainer.__init__(self,initdict=data2) # Set all content to attributes on self


class Label(GenericDataContainer.GenericDataContainer):
    def __init__(self, name, id, data=None):
        """
        Corresponds to a Label for a Board in Projectplace
        @param name: Name of Label
        @type name: '<name>'
        @param id: Id of Label
        @type id: <id>
        @param data: Data retrieved from Projectplace API representing a Label
        @type data: {'name':'<name>','id':<id>, ...} [primary data, more will be available]
        """
        self.name = name
        self.id = id
        
        if data:
            data2 = data.copy()
            del data2['id']     # We don't want these to overwrite the self.id
            del data2['name']  # We don't want this to overwrite the self.name
            GenericDataContainer.GenericDataContainer.__init__(self,initdict=data2)# Set all content to attributes on self


class Card(GenericDataContainer.GenericDataContainer):
    def __init__(self, name, id, data=None):
        """
        Corresponds to a Card in Projectplace
        @param name: Name of Card
        @type name: '<name>'
        @param id: Id of Card
        @type id: <id>
        @param data: Data retrieved from Projectplace API representing a Card
        @type data: {'name':'<name>','id':<id>, ...} [primary data, more will be available]
        """
        self.name = name
        self.id = id
        
        if data:
            data2 = data.copy()
            del data2['id']     # We don't want these to overwrite the self.id
            #del data2['name']  # There is no "name" key, only "title".
            GenericDataContainer.GenericDataContainer.__init__(self,initdict=data2) # Set all content to attributes on self

        self._status = None
        self._label = None
        
        
    @property
    def status(self):
        """
        Return status
        """
        if not self._status:
            #Todo: Fetch data from PP API if self.progress doesn't exist
            self._status = Status(id=self.progress['id'], name=self.progress['name'], data=self.progress)
        return self._status


    def set_status(self, status):
        """
        Set Status of card in Board
        @param status: The (new) status that the card should have
        @type status: <instance of Status>
        """
        from ppcardhandler.core import set_card_status
        updated = set_card_status(self.id, status.id)   # Update on server
        self._status = Status(id=updated['progress']['id'], name=updated['progress']['name'], data=updated['progress']) # Update self


    def set_description(self, description):
        """
        Set description of card in Board
        @param description: The (new) description that the card should have
        @type status: '<value>'
        """
        from ppcardhandler.core import set_card_description
        set_card_description(self.id, description)
        self.description = description


    @property
    def label(self):
        """
        Return label
        """
        if not self._label:
            if self.label_id == None:
                self._label = Label(id=None, name='', data={})
            else:
                from ppcardhandler.core import get_board_label
                labeldata = get_board_label(boardid=self.board_id, labelid=self.label_id)
                self._label = Label(id=labeldata['id'], name=labeldata['name'], data=labeldata)
        return self._label


    def set_label(self, label):
        """
        Set label of a Card in Board

        @param label: The (new) label that the card should have
        @type label: <instance of Label>
        """
        from ppcardhandler.core import set_card_label, get_board_label
        updated = set_card_label(self.id, label.id) # Update on server
        labeldata = get_board_label(boardid=self.board_id, labelid=updated['label_id']) # Get label name etc.
        self._label = Label(id=labeldata['id'], name=labeldata['name'], data=labeldata) # Update self
        
        
class Board(GenericDataContainer.GenericDataContainer):
    def __init__(self, projectid, name, id, data=None):
        """
        Corresponds to a Board in Projectplace
        @param projectid: Id of Project that Board belongs to (needed for creation of new cards via API)
        @type projectid: <project id>
        @param name: Name of Board
        @type name: '<name>'
        @param id: Id of Board
        @type id: <board id>
        @param data: Data retrieved from Projectplace API representing a Board
        @type data: {'name':'<name>','id':<id>, ...} [primary data, more will be available]
        """
        
        self.projectid = projectid
        self.name = name
        self.id = id

        if data:
            data2 = data.copy()
            del data2['id']     # We don't want these to overwrite the self.id and self.name
            del data2['name']
            if 'labels' in data2:
                # Convert data into a objects to provide functionality, replace with "_label" to enable use of @property
                data2['_labels'] = [Label(ldata['name'],ldata['id'],data=ldata) for ldata in data2['labels']]
                del data2['labels']
            else:
                data2['_labels'] = None
            GenericDataContainer.GenericDataContainer.__init__(self,initdict=data2) # Set all content to attributes on self
            
        self._cards = None
        self._statuses = None

        self.__raw_properties_from_api = None # Cache mechanism when we do not get content in data (at some point it will be fetched)        
        
                
        
    @property
    def cards(self):
        """
        Return Cards associated with Board
        """
        if self._cards == None:
            # Cards info not yet retrieved from API, do it
            from ppcardhandler.core import get_cards
            data = get_cards(self.id)
            self._cards = dict()
            for entry in data:
                self._cards[entry['id']] = Card(name=entry['title'], id=entry['id'], data=entry)
                
        return self._cards

    def card(self, id=0, name='', create=False):
        """
        Return requested Card
        @param id: Card id [optional]
        @type id: <id>
        @param name: Card name [optional]
        @type name: '<name>'
        @param create: If Card should be created (if it doesn't exist)
        @type create. <Boolean>
        @return: A board
        """
        if id:
            # Fetch existing board by ID
            return self.cards[id]
        
        else:
            # Fetch existing board by NAME
            chkname = name.lower()
            for id in self.cards:
                if self.cards[id].name.lower() == chkname:
                    return self.cards[id]
            # Create card
            if create:
                from ppcardhandler.core import create_card
                entry_data = create_card(projectid=self.projectid, boardid=self.id, name=name)
                self._cards[entry_data['id']] = Card(name=entry_data['title'], id=entry_data['id'], data=entry_data)
                return self._cards[entry_data['id']]
            
            # Nothing to return
            else:
                return None


    @property
    def _raw_properties_from_api(self):
        """
        If self.__init__ is not provided with teh data parameter then
        we need to fetch data from the Projectplace API later on.
        This will fetch it and cache it on the object so other
        properties can make use of teh same data rather than having to
        make a new call to the Projectplace API.
        """
        from ppcardhandler.core import get_board
        if not self.__raw_properties_from_api:
            self.__raw_properties_from_api = get_board(boardid=self.id)
            print self.__raw_properties_from_api
        return self.__raw_properties_from_api

    
    @property 
    def statuses(self):
        """
        Return all available Statuses for this board
        @return: List of Statuses
        @note: we use "status" terminology instead of the "progress" terminology of the PP API
        """
        if self._statuses == None:
            # Organise status data in more structured format
            self._statuses = list()
            if hasattr(self, 'progresses'): # If __init__ was provided with values in "data" property
                for entry in self.progresses:
                    self._statuses.append(Status(name=entry['name'], id=entry['id'], data=entry))
                del self.progresses     # delete to prevent confusion.
            else:
                # Get data from PP API (or use cached data from previously done request)
                for entry in self._raw_properties_from_api['progresses']:
                    self._labels.append(Status(name=entry['name'], id=entry['id'], data=entry))

        return self._statuses
    

    def status(self, name, exactmatchonly=True):
        """
        Return status
        @param name: Name of Status
        @type name: '<name>'
        @param exactmatchonly: Option to require less than exact match
        @type exactmatchonly: <Boolean>
        @return: <instance of Status with a corresponding name> or None
        """
          
        for status in self.statuses:
            if status.identifier_matches(name, exactmatchonly=exactmatchonly):
                return status
        return None

    
    @property
    def labels(self):
        """
        Return all available Labels for this board
        @return: [<instance of Label>,]
        """
        if self._labels == None:
            # Organise labels data in more structured format
            self._labels = list()
            for entry in self._raw_properties_from_api['labels']:
                self._labels.append(Label(name=entry['name'], id=entry['id'], data=entry))

        return self._labels


    def label(self, name, exactmatchonly=True):
        """
        Return label
        @param name: Name of Label
        @type name: '<name>'
        @param exactmatchonly: Option to require less than exact match
        @type exactmatchonly: <Boolean>
        @return: <instance of Label with a corresponding name> or None
        """
          
        for label in self.labels:
            if label.identifier_matches(name, exactmatchonly=exactmatchonly):
                return label
        return None
        

class Project(GenericDataContainer.GenericDataContainer):
    def __init__(self, name=None, id=None, data=None):
        """
        Corresponds to a project in Projectplace
        @param name: Name of Project
        @type name: '<name>'
        @param id: Id of Project (if None then will be taken from "data" or fetched from Projectplace)
        @type id: <id>
        @param data: Data representing a project. None to fetch from API, {} to not fetch.
        @type data: {'name':'<name>','id':<id>, ...} [primary data, more will be available]
        """

        if name==None and id==None:
            raise ValueError('"name" or "id" must be provided')


        if data==None:
            # Regarding data: We will fetch if data is None. In case of an empty dict
            # we do not fetch (assumes there is no need/use for the data). This can
            # be utilized to speed up init process. If you have no
            # need for the metadata (description, etc.) but are
            # interested in Boards & Cards methods then use data={}
            
            # Fetch meta data for Project from API.
            from ppapiaccess import get_projects
            
            data = get_projects(name=name, id=id)
            if not data:
                errormsg = """No Project could be fetched from Projectplace.com. \
Neither supplied name ("%s") or id ("%s") correspond to any \
accessible Project. Please verify these values and try again."""%(name, id)
                raise ValueError(errormsg)
            
            if not id:
                id = data['id']
            if not name:
                name = data['name']
            
        self.name = name
        self._id = id

        if data:
            # Populate current object with attributes corresponding to content in data
            data2 = data.copy()
            
            # We don't want to overwrite the self.id and self.name (provided they have a value)
            if not self._id:
                data2['_id'] = data2['id']
            del data2['id']     
            del data2['name']

            # Call parent                        
            GenericDataContainer.GenericDataContainer.__init__(self,initdict=data2)
        
        self._boards = None

    @property
    def id(self):
        """
        Return project id
        @return: <number>
        """
        if not self._id:
            # Fetch data from PP API
            self.__init__(self.name, id=None, data=None)
        return self._id 

    @property
    def boards(self):
        """
        Return Boards associated with project
        """
        if self._boards == None:
            # Boards info not yet retrieved from API, do it
            from ppcardhandler.core import get_boards
            data = get_boards(self.id)
            self._boards = dict()
            for entry in data:
                new_board = Board(projectid=self.id, name=entry['name'], id=entry['id'], data=entry)
                self._boards[new_board.id] = new_board
                
        return self._boards

                
    def board(self, id=0, name='', create=False):
        """
        Return requested board
        @param id: Board id [optional]
        @type id: <id>
        @param name: Board name [optional]
        @type name: '<name>'
        @param create: If board should be created (if it doesn't exist)
        @type create. <Boolean>
        @return: A board
        """
        if id:
            # Fetch existing board by ID
            return self.boards[id]
        
        else:
            # Fetch existing board by NAME
            chkname = name.lower()
            for id in self.boards:
                if self.boards[id].name.lower() == chkname:
                    return self.boards[id]
            # Create board
            if create:
                from ppcardhandler.core import create_board
                entry_data = create_board(self.id, name)
                self._boards[entry_data['id']] = Board(projectid=self.id, name=entry_data['name'], id=entry_data['id'], data=entry_data)
                return self._boards[entry_data['id']]
            
            # Nothing to return
            else:
                return None



