
def get_boards(projectid):
    """
    Get (attribuets for) available Boards for a single Project
    @param projectid: Project id
    @type projectid: <id>
    @return: [{board attributes},]
    """
    from ppapiaccess import connect
    
    if not projectid:
        raise ValueError('Parameter projectid must contain value')
    
    conn = connect()
    response = conn.request('GET','/1/project/%d/boards.json'%int(projectid))
    if response.status_code != 200:
        raise RuntimeError('Error occured. Server returned http status code %d. Return message: %s'%(response.status_code, response.text))

    data = response.json()
    return data


def get_board(boardid):
    """
    Get attributes for a single Board
    @param boardid: Board id
    @type boardid: <id>
    @return: {board attributes}
    """
    from ppapiaccess import connect
    
    if not boardid:
        raise ValueError('Parameter boardid must contain value')
    
    conn = connect()
    response = conn.request('GET','/1/board/%d/properties.json'%int(boardid))
    if response.status_code != 200:
        raise RuntimeError('Error occured. Server returned http status code %d. Return message: %s'%(response.status_code, response.text))

    data = response.json()
    return data


def create_board(projectid, name):
    """
    Create a new board
    @param projectid: Id of project to create board in
    @type projectid: <id>
    @param name: Name of board to be created
    @type name: '<name>'
    @return: Board data
    """
    from ppapiaccess import connect
    
    if not projectid:
        raise ValueError('Parameter projectid must be supplied')
    
    conn = connect()
    response = conn.request('POST','/1/project/%d/boards/create-new.json'%int(projectid),{'name':name})
    if response.status_code != 200:
        raise RuntimeError('Error occured. Server returned http status code %d. Return message: %s'%(response.status_code, response.text))

    data = response.json()
    return data



def get_cards(boardid):
    """
    Get available cards on a specific board
    @param boardid: Id of the board
    @type boardid: <id>
    @returns cards
    @note The terminology is confusing. The public terminology is "Card" but in API it's "Activity".
    """
    from ppapiaccess import connect
    
    if not boardid:
        raise ValueError('Parameter boardid must be supplied')
    
    conn = connect()
    response = conn.request('GET','/1/board/%d/activities.json'%int(boardid))
    if response.status_code != 200:
        raise RuntimeError('Error occured. Server returned http status code %d. Return message: %s'%(response.status_code, response.text))

    data = response.json()
    return data


def create_card(projectid, boardid, name):
    """
    Create a new Card
    @param projectid: Id of project to card in
    @type projectid: <id>
    @param boardid: Id of board to create card in
    @type boardid: <id>
    @param name: Name of card to be created
    @type name: '<name>'
    @return: card data
    """
    from ppapiaccess import connect
    
    if not (projectid and boardid and name):
        raise ValueError('Parameters projectid, boardid and name must be supplied')
    
    conn = connect()
    response = conn.request('POST','/1/project/%d/activities/create-new.json'%int(projectid),{'title':name, 'board_id':boardid})
    if response.status_code != 200:
        raise RuntimeError('Error occured. Server returned http status code %d. Return message: %s'%(response.status_code, response.text))

    data = response.json()
    return data



def _set_card_property(cardid, data):
    """
    Will update card property on server, can be one or several properties/attributes

    @param cardid: Id of card to update
    @type cardid: <id>
    @param data: Properties/attributes to set
    @type data: {'<property/attribute name>':<value>,}
    @return: {'<property/attribute name>':<value>,} all values for Card after update
    """

    from ppapiaccess import connect
    
    if not (cardid and data):
        raise ValueError('Parameters "cardid" and "data" must be supplied')
    
    conn = connect()
    url = '/1/activity/%d/properties.json'%int(cardid)
    response = conn.request('POST','/1/activity/%d/properties.json'%int(cardid),data)
    if response.status_code != 200:
        raise RuntimeError('Error occured. URL(post): %s. Values(data): %s. Server returned http status code %d. Return message: %s'%(url, str(data), response.status_code, response.text))

    data = response.json()
    return data


def set_card_description(cardid, description):
    """
    Set description of a Card
    
    @param cardid: Id of card to update
    @type cardid: <id>
    @param description: Description of card to be set
    @type statusid: '<description>'
    @return: {'<property/attribute name>':<value>,} all values for card after update
    """
    if not (cardid and description):
        raise ValueError('Parameters cardid and description must be supplied')
    return _set_card_property(cardid, {'description':description})


def set_card_status(cardid, statusid):
    """
    Set status of a Card
    
    @param cardid: Id of card to update
    @type cardid: <id>
    @param statusid: Id of status to be set
    @type statusid: <id>
    @return: {'<property/attribute name>':<value>,} all values for card after update
    """
    if not (cardid and statusid):
        raise ValueError('Parameters cardid and statusid must be supplied')
    return _set_card_property(cardid, {'progress_id':statusid})
    


def get_board_label(boardid, labelid):
    """
    Get label (a label is spcific for each Board)
    
    @param boardid: Board id
    @type boardid: <id>
    @param labelid: Label id
    @type labelid: <id>
    @return: {label attributes}
    """
    from ppapiaccess import connect
    
    if not (boardid) or labelid==None:   # Special case, labelids (index) starts on 0 , thus 0 is ok value
        raise ValueError('Parameters boardid and labelid must contain values')
    
    conn = connect()
    url = '/1/board/%d/label/%d/properties.json'%(int(boardid), int(labelid))
    response = conn.request('GET',url)
    if response.status_code != 200:
        raise RuntimeError('Error occured (url: %s). Server returned http status code %d. Return message: %s'%(url, response.status_code, response.text))

    data = response.json()
    return data


def set_card_label(cardid, labelid):
    """
    Set label for a Card
    
    @param cardid: Id of card to update
    @type cardid: <id>
    @param labelid: Label id
    @type labelid: <id>
    @@return: {'<property/attribute name>':<value>,} all values for card after update
    """
    
    if not (cardid) or labelid==None:   # Special case, labelids (index) starts on 0 , thus 0 is ok value
        raise ValueError('Parameters cardid and labelid must be supplied')
    return _set_card_property(cardid, {'label_id':labelid})
